#include "uart.hpp"
#include <stm32f0xx.h>

using namespace stm32f072xb;

///////////////////////
// UART1, PORTA
///////////////////////

Uart<UartRegister::UART1, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART1, UartGpioType::TX, Port::A> txGpio)
    : AbstractUart(USART1, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);  // enable uart1 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART1, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART1, UartGpioType::RX, Port::A> rxGpio)
    : AbstractUart(USART1, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);  // enable uart1 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART1, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART1, UartGpioType::TX, Port::A> txGpio,
                                         UartGpio<UartRegister::UART1, UartGpioType::RX, Port::A> rxGpio)
    : AbstractUart(USART1, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);  // enable uart1 (p.116)
	this->setupRegister();
}

///////////////////////
// UART1, PORTB
///////////////////////

Uart<UartRegister::UART1, Port::B>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART1, UartGpioType::TX, Port::B> txGpio)
    : AbstractUart(USART1, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);  // enable uart1 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART1, Port::B>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART1, UartGpioType::RX, Port::B> rxGpio)
    : AbstractUart(USART1, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);  // enable uart1 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART1, Port::B>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART1, UartGpioType::TX, Port::B> txGpio,
                                         UartGpio<UartRegister::UART1, UartGpioType::RX, Port::B> rxGpio)
    : AbstractUart(USART1, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB2ENR, RCC_APB2ENR_USART1EN);  // enable uart1 (p.116)
	this->setupRegister();
}

///////////////////////
// UART2, PORTA
///////////////////////

Uart<UartRegister::UART2, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART2, UartGpioType::TX, Port::A> txGpio)
    : AbstractUart(USART2, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART2, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART2, UartGpioType::RX, Port::A> rxGpio)
    : AbstractUart(USART2, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN);
	this->setupRegister();
}

Uart<UartRegister::UART2, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART2, UartGpioType::TX, Port::A> txGpio,
                                         UartGpio<UartRegister::UART2, UartGpioType::RX, Port::A> rxGpio)
    : AbstractUart(USART2, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN);  // enable uart2 (p.116)
	this->setupRegister();
}

///////////////////////
// UART2, PORTD
///////////////////////

Uart<UartRegister::UART2, Port::D>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART2, UartGpioType::TX, Port::D> txGpio)
    : AbstractUart(USART2, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART2, Port::D>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART2, UartGpioType::RX, Port::D> rxGpio)
    : AbstractUart(USART2, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART2, Port::D>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART2, UartGpioType::TX, Port::D> txGpio,
                                         UartGpio<UartRegister::UART2, UartGpioType::RX, Port::D> rxGpio)
    : AbstractUart(USART2, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART2EN);  // enable uart2 (p.116)
	this->setupRegister();
}

///////////////////////
// UART3, PORTB
///////////////////////
Uart<UartRegister::UART3, Port::B>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::TX, Port::B> txGpio)
    : AbstractUart(USART3, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART3, Port::B>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::RX, Port::B> rxGpio)
    : AbstractUart(USART3, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART3, Port::B>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::TX, Port::B> txGpio,
                                         UartGpio<UartRegister::UART3, UartGpioType::RX, Port::B> rxGpio)
    : AbstractUart(USART3, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

///////////////////////
// UART3, PORTC
///////////////////////

Uart<UartRegister::UART3, Port::C>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::TX, Port::C> txGpio)
    : AbstractUart(USART3, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART3, Port::C>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::RX, Port::C> rxGpio)
    : AbstractUart(USART3, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART3, Port::C>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::TX, Port::C> txGpio,
                                         UartGpio<UartRegister::UART3, UartGpioType::RX, Port::C> rxGpio)
    : AbstractUart(USART3, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

///////////////////////
// UART3, PORTD
///////////////////////

Uart<UartRegister::UART3, Port::D>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::TX, Port::D> txGpio)
    : AbstractUart(USART3, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART3, Port::D>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::RX, Port::D> rxGpio)
    : AbstractUart(USART3, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART3, Port::D>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART3, UartGpioType::TX, Port::D> txGpio,
                                         UartGpio<UartRegister::UART3, UartGpioType::RX, Port::D> rxGpio)
    : AbstractUart(USART3, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART3EN);  // enable uart3 (p.116)
	this->setupRegister();
}

///////////////////////
// UART4, PORTA
///////////////////////

Uart<UartRegister::UART4, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART4, UartGpioType::TX, Port::A> txGpio)
    : AbstractUart(USART4, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART4EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART4, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART4, UartGpioType::RX, Port::A> rxGpio)
    : AbstractUart(USART4, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART4EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART4, Port::A>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART4, UartGpioType::TX, Port::A> txGpio,
                                         UartGpio<UartRegister::UART4, UartGpioType::RX, Port::A> rxGpio)
    : AbstractUart(USART4, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART4EN);  // enable uart2 (p.116)
	this->setupRegister();
}

///////////////////////
// UART4, PORTC
///////////////////////

Uart<UartRegister::UART4, Port::C>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART4, UartGpioType::TX, Port::C> txGpio)
    : AbstractUart(USART4, uartInit), _txGpio(txGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART4EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART4, Port::C>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART4, UartGpioType::RX, Port::C> rxGpio)
    : AbstractUart(USART4, uartInit), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART4EN);  // enable uart2 (p.116)
	this->setupRegister();
}

Uart<UartRegister::UART4, Port::C>::Uart(AbstractUartInit uartInit,
                                         UartGpio<UartRegister::UART4, UartGpioType::TX, Port::C> txGpio,
                                         UartGpio<UartRegister::UART4, UartGpioType::RX, Port::C> rxGpio)
    : AbstractUart(USART4, uartInit), _txGpio(txGpio), _rxGpio(rxGpio) {
	SET_BIT(RCC->APB1ENR, RCC_APB1ENR_USART4EN);  // enable uart2 (p.116)
	this->setupRegister();
}
